
/*
 * Static data and loader exports.
 */

#include <sys/conf.h>
#include <sys/cmn_err.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/ksynch.h>
#include <sys/kmem.h>
#include <sys/modctl.h>

#include "defs.h"
#include "rbuffer.h"
#include "drv.h"


/*  cb_ops structure.
    See Writing Device Drivers or 'man cb_ops' for this one  */
static struct cb_ops drv_cb_ops = 
{
	drv_open,
	drv_close,
	nodev,
	nodev,
	nodev,
	drv_read,
	drv_write,
	drv_ioctl,
	nodev,
	nodev,
	nodev,
	drv_poll,
	ddi_prop_op,
	NULL,
	D_MP | D_NEW,
	CB_REV,
	nodev,
	nodev
};


/*  dev_ops structure.
    See Writing Device Drivers or 'man dev_ops' for this one  */
static struct dev_ops drv_dev_ops = 
{
	DEVO_REV,
	0,
	drv_getinfo,
	nulldev,
	drv_probe,
	drv_attach,
	drv_detach,
	nodev,
	&drv_cb_ops,
	NULL,
	nodev
};


/*  modldrv structure.
    See Writing Device Drivers or 'man modldrv' for this one  */
static struct modldrv drv_modldrv = 
{
	&mod_driverops,
	DRV_NAME,
	&drv_dev_ops
};


/*  modlinkage structure.
    See Writing Device Drivers or 'man modlinkage' for this one  */
static struct modlinkage drv_modlinkage =
{
	MODREV_1,
	&drv_modldrv,
	NULL
};


/*  Soft state management  */
void* drv_soft_state;


/*  Access attributes
    See Writing Device Drivers book, or the manpage.  */
ddi_device_acc_attr_t drv_acc_attr = 
{
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_LE_ACC,
	DDI_STRICTORDER_ACC
};


/*  Module startup routine.
    See 'man _init' for more info about it.  */
int _init(void)
{
	int ret;
	
	ret = ddi_soft_state_init(&drv_soft_state, sizeof(devstate_t), 1);
	if (ret == 0) {
		ret = mod_install(&drv_modlinkage);

		if (ret != 0) {
			ddi_soft_state_fini(&drv_soft_state);
		}
	}

	if (ret == 0)
		cmn_err(CE_NOTE, "opto: driver version 1.0 loaded.");

	return (ret);
}


/*  Module shutdown routine.
    See 'man _fini' for more info about it. */
int _fini(void)
{
	int ret;

	ret = mod_remove(&drv_modlinkage);
	if (ret == 0) {
		ddi_soft_state_fini(&drv_soft_state);
	}

	if (ret == 0)
		cmn_err(CE_NOTE, "opto: driver shutdown.");

	return (ret);
}


/*  Info routine. This one is called when you type
    'modinfo', for example. See 'man _info' for more info about it.  */
int _info(struct modinfo* mi)
{
	return  mod_info(&drv_modlinkage, mi);
}

