
/*
 *
 * Hardware interface
 *
 */

#ifndef __DRV_H_
#define __DRV_H_


#include "defs.h"
#include "rbuffer.h"
#include "opto.h"


/*  Compiled-in default values for timer modes and
    start values  */
#define CNT0_DEFSVAL  0x0004
#define CNT1_DEFSVAL  0x0014
#define CNT2_DEFSVAL  0x000a

#define CNT0_DEFMODE  OPTO_CNTMODE_SQUAREGEN
#define CNT1_DEFMODE  OPTO_CNTMODE_SQUAREGEN
#define CNT2_DEFMODE  OPTO_CNTMODE_NDIV

#define DEFINTRMASK   0x1

/* Hardware registers found on OPTOIN-16 extended boards  */
struct oin16regs
{
	byte_t in00_07; /* input register, inputs 0-7 */
	byte_t in08_15; /* input register, inputs 8-15 */
  
	byte_t unused1; /* 6-byte unused area */
	byte_t unused2; 
	byte_t unused3;
	byte_t unused4;
	byte_t unused5;
	byte_t unused6;

	byte_t cnt0; /* counter 0, read-write buffer */
	byte_t cnt1; /* counter 1, read-write buffer */
	byte_t cnt2; /* counter 3, read-write buffer */
  
	byte_t ctrl; /* control register */

	byte_t irrisr; /* PIC IRR-ISR register */
	byte_t imr; /* PIC IRM register */
};
typedef struct oin16regs oin16regs_t;


/*  Device state structure. There is one instance of
    this per device.  */
struct devstate
{
	dev_info_t* dip;
	int inst;
	int refc;
	byte_t intr_mask;

	word_t prev_val;
	word_t cur_val;
	
	ddi_iblock_cookie_t ibc;
	ddi_acc_handle_t ah;
	struct pollhead ph;

	kmutex_t mtx;

	oin16regs_t* regs;
	rbuffer_t* rbuf;
};
typedef struct devstate devstate_t;


/*  DDI Charracter device IO  */
int drv_open(dev_t* dev, int oflags, int otype, cred_t* cred);
int drv_close(dev_t dev, int oflags, int otype, cred_t* cred);
int drv_read(dev_t dev, struct uio* uiop, cred_t* cred);
int drv_write(dev_t dev, struct uio* uiop, cred_t* cred);
int drv_poll(dev_t dev, short events, int anyyet, short* revents, struct pollhead** ph);
int drv_ioctl(dev_t dev, int cmd, intptr_t arg, int mode, cred_t* cred, int* rval);


/*  DDI entry points for autoconfiguration  */
int drv_attach(dev_info_t* dip, ddi_attach_cmd_t cmd);
int drv_detach(dev_info_t* dip, ddi_detach_cmd_t cmd);
int drv_probe(dev_info_t* dip);
int drv_getinfo(dev_info_t* dip, ddi_info_cmd_t cmd, void* arg, void** res);


/*  Interrupt handler  */
u_int drv_intrh(caddr_t inst);


/*  Device IO, direct hardware access  */
void pic8259a_init(devstate_t* state, byte_t ltim);
void pic8259a_set_intr_mask(devstate_t* state, byte_t im);
void cnt8254_init(devstate_t* state, byte_t tnum, byte_t tmode, word_t stval);


/*  Global data  */
extern void* drv_soft_state;
extern ddi_device_acc_attr_t drv_acc_attr; 


#endif /* __DRV_H_ */

