
/*
 * Interrupt handling
 *
 */

#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "defs.h"
#include "rbuffer.h"
#include "drv.h"


/*  Interrupt handler  */
u_int drv_intrh(caddr_t inst)
{
	int has_input = 0;
	byte_t lo, hi;
	devstate_t* state;
	
	state = ddi_get_soft_state(drv_soft_state, (int)inst);
	if (state == NULL)
		return (DDI_INTR_UNCLAIMED);

	mutex_enter(&state->mtx);

	lo = ddi_get8(state->ah, &(state->regs->in00_07));
	hi = ddi_get8(state->ah, &(state->regs->in08_15));
	
	state->cur_val = MAKEWORD(lo, hi);

	if (state->cur_val != state->prev_val) {
		state->prev_val = state->cur_val;
		has_input = 1;
		
		state->rbuf->store(state->rbuf, lo);
		state->rbuf->store(state->rbuf, hi);
	}
	
	mutex_exit(&state->mtx);

	/*  Calling pollwakeup() while holding a mutex can
	    cause bad things to happen. We call it after the mutex
	    has been released.  */
	if (has_input == 1)
		pollwakeup(&state->ph, POLLIN);

	return (DDI_INTR_CLAIMED);
}

