
/*
 *     IOCTL interface for the opto driver.
 *     ===================================
 *
 *     Please consult the opto driver user's guide for detailed 
 *     information regarding the driver ioctl interface and related
 *     issues.
 *
 *     Copyright (C) 2000 H.A.N.D. GmbH, All Rights Reserved.
 *     Please consult your license for further information regarding
 *     allowances and restrictions of this code.
 */

#ifndef __OPTO_H_
#define __OPTO_H_


#define OPTO_DEVICECHAR 'o'
#define OPTO_CTL(_x) ((OPTO_DEVICECHAR << 8) | (_x))
#define OPTO_REG_IS_VALID(_x) \
    ( ((_x) == 0x0) || ((_x) == 0x1) || (((_x) >= 0x8) && ((_x) <= 0xd)) )
#define OPTO_CNT_IS_VALID(_x) \
    ( ((_x) == 0x0) || ((_x) == 0x1) || ((_x) == 0x2) )


/*  IOCTL commands  */
#define OPTOCTL_GET_INTRMASK OPTO_CTL(1)
#define OPTOCTL_SET_INTRMASK OPTO_CTL(2)
#define OPTOCTL_READ_REG     OPTO_CTL(3)
#define OPTOCTL_WRITE_REG    OPTO_CTL(4)
#define OPTOCTL_CNT_INIT     OPTO_CTL(5)
#define OPTOCTL_PIC_INIT     OPTO_CTL(6)


/*  OPTOIN-16 register names  */
#define OPTOIN16_REG_IN0    0x0
#define OPTOIN16_REG_IN1    0x1
#define OPTOIN16_REG_CNT0   0x8
#define OPTOIN16_REG_CNT1   0x9
#define OPTOIN16_REG_CNT2   0xa
#define OPTOIN16_REG_CTRL   0xb
#define OPTOIN16_REG_IRRISR 0xc
#define OPTOIN16_REG_IMR    0xd


/*  PIC level trigger mode  */
#define OPTO_LTIM_HIGH  0x00
#define OPTO_LTIM_FLANK 0x01


/*  Timer operation modes  */
#define OPTO_CNTMODE_ZEROTHROUGH   0x00
#define OPTO_CNTMODE_MONOSTABLE    0x01
#define OPTO_CNTMODE_NDIV          0x02
#define OPTO_CNTMODE_SQUAREGEN     0x03
#define OPTO_CNTMODE_SOFTWARE      0x04
#define OPTO_CNTMODE_HARDWARE      0x05


/*  Counter initialization structure  */
typedef struct opto_cnt
{
	unsigned char num;
	unsigned char mode;
	int stval;
}
opto_cnt_t;

/*  PIC initialization structure  */
typedef struct opto_reg
{
	unsigned char reg;
	unsigned char val;
}
opto_reg_t;


/*  Inlines and macros which should make the life a bit easier
    to these ioctl callers  */
#ifdef __cplusplus

inline int opto_get_intrmask(int fd, unsigned char* im) 
    { return ioctl(fd, OPTOCTL_GET_INTRMASK, im); };
inline int opto_set_intrmask(int fd, unsigned char* im) 
    { return ioctl(fd, OPTOCTL_SET_INTRMASK, im); };
inline int opto_read_reg(int fd, opto_reg_t* ori) 
    { return ioctl(fd, OPTOCTL_READ_REG, r); };
inline int opto_write_reg(int fd, opto_reg_t* ori) 
    { return ioctl(fd, OPTOCTL_WRITE_REG, r); };
inline int opto_cnt_init(int fd, opto_cnt_t* oci)
    { return ioctl(fd, OPTOCTL_CNT_INIT, oci); };
inline int opto_pic_init(int fd, unsigned char* ltim)
    { return ioctl(fd, OPTOCTL_PIC_INIT, ltim); };

#else /*   not c++  */

#define opto_get_intrmask(_f, _p) ioctl((_f), OPTOCTL_GET_INTRMASK, (_p))
#define opto_set_intrmask(_f, _p) ioctl((_f), OPTOCTL_SET_INTRMASK, (_p))
#define opto_read_reg(_f, _p) ioctl((_f), OPTOCTL_READ_REG, (_p))
#define opto_write_reg(_f, _p) ioctl((_f), OPTOCTL_WRITE_REG, (_p))
#define opto_cnt_init(_f, _p) ioctl((_f), OPTOCTL_CNT_INIT, (_p))
#define opto_pic_init(_f, _p) ioctl((_f), OPTOCTL_PIC_INIT, (_p))

#endif /*  if c++  */


#endif __OPTO_H_
