
/*
 *  optostress utility
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <poll.h>
#include "opto.h"


#define DEFDEV "/dev/opto0"

#define DO_NOTHING    0x0
#define DO_HELP       0x1
#define DO_READ       0x2
#define DO_WRITE      0x4
#define DO_POLL       0x8
#define DO_CONTINUOUS 0x10
#define DO_BADPARAM   0x20

#define COL_WIDTH 66

typedef struct action {
	int todo;
	char* device;
	char* source;
	char bad_param;
	size_t amount;
} action_t;


/*  Display help screen  */
void display_help(void)
{
	char help[] =
		"\n"
		"optostress utility, version 1.0\n"
		"calling syntax not determined yet.\n\n";


	printf(help);
	return;
}


/* Polls and displays the incoming data on the screen
   until interrupted.  */
void poll_device(action_t* act)
{
	struct pollfd fds = {0, POLLIN, 0};
	unsigned char data;
	int bytec = 0;

	fds.fd = open(act->device, O_RDONLY);
	if (fds.fd == 0)
		return;

	while (1) {
		poll(&fds, 1, (-1));
		while (read(fds.fd, &data, sizeof(unsigned char)) > 0) {
			printf("%02X ", data);
			bytec++;
			if (3*bytec > COL_WIDTH) {
				printf("\n");
				bytec = 0;
			}
		}
	}
}


/* Creates new action structure  */
action_t* action_create(void)
{
	action_t* act;

	act = (action_t*)malloc(sizeof(action_t));
	if (act != NULL) {
		act->todo = DO_NOTHING;
		act->device = DEFDEV;
		act->amount = 0;
		act->bad_param = '\0';
	}

	return (act);
}


/* destroys the action */
void action_destroy(action_t* act)
{
	free(act);
	return;
}


/* performs an action */
int action_perform(action_t* act)
{
	int ret = -1;

	if (act->todo & DO_HELP || act->todo == DO_NOTHING)
		display_help();

	if (act->todo & DO_BADPARAM)
		printf("optostress: Bad Parameter '%c'.\n", act->bad_param);

	if (act->todo & DO_POLL)
		poll_device(act);

	return ret;
}


/*  Sets up an action from the command line  */
void parse_cmdline(action_t* act, int argv, char** argc)
{
	int i;
	char* walk;

	for (i = 1; i < argv; i++) {
		walk = argc[i];
		if (act->todo & DO_BADPARAM)
			break;
		while (*walk != '\0') {
			switch (*walk) {
			/* whitespace */
			case ' ':
			case '-':
				break;

			case 'p':
				act->todo |= DO_POLL;
				break;
			case 'c':
				act->todo |= DO_CONTINUOUS;
				break;
			case 'h':
				act->todo |= DO_HELP;
				break;
			default:
				act->todo |= DO_BADPARAM;
				act->bad_param = *walk;
			}
			walk++;
		}
	}
}


/*  Main entry point  */
int main(int argv, char** argc)
{
	action_t* act;

	act = action_create();
	if (act == NULL)
		return (127);

	parse_cmdline(act, argv, argc);
	action_perform(act);
	action_destroy(act);

	return (0);
}

